package com.tptj.demo.hg.template.encrypt;

import com.fr.general.CommonIOUtils;
import com.fr.report.fun.impl.AbstractTemplateEncryptProvider;
import com.fr.security.SecurityToolbox;
import com.fr.stable.CodeUtils;
import com.fr.stable.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021/9/12
 **/
public class DemoEncryptor extends AbstractTemplateEncryptProvider {

    public static final String TAG_START = "-----Encrypt Start-----";
    public static final String TAG_END = "-----Encrypt End-----";

    public static ThreadLocal<String> holder = new ThreadLocal<String>();
    public static ThreadLocal<Integer> pos = new ThreadLocal<Integer>();
    public static ThreadLocal<Boolean> init = new ThreadLocal<Boolean>();

    public static void setSecret(String secret) {
        holder.set(secret);
    }

    public static String getSecret() {
        return holder.get();
    }

    public static boolean hasSecret() {
        return StringUtils.isNotEmpty(getSecret());
    }

    public static int getPos(){
        return pos.get();
    }

    public static void setPos(int idx){
        pos.set(idx);
    }

    public static void initData(InputStream in){
        init.set(true);
        String data = StringUtils.EMPTY;
        try{
            data = CommonIOUtils.inputStream2String(in);
            if(!data.startsWith(TAG_START)){
                setSecret(StringUtils.EMPTY);
                setPos(-1);
                return;
            }
            int end = data.indexOf(TAG_END);
            String part = data.substring(TAG_START.length(),end);
            String sha = part.substring(0,64);
            String code = part.substring(64);
            String secret = CodeUtils.passwordDecode("___"+code);
            if( !CodeUtils.md5Encode(secret, StringUtils.EMPTY,"SHA-256").equals(sha) ){
                setSecret(StringUtils.EMPTY);
                setPos(-1);
                return;
            }
            setSecret(secret);
            setPos(end+TAG_END.length());
        }catch(Exception e){
            setSecret(StringUtils.EMPTY);
            setPos(-1);
        }
    }

    private String codePassword(){
        String secret = getSecret();
        String code = CodeUtils.passwordEncode(secret).substring(3);
        String sha = CodeUtils.md5Encode(secret, StringUtils.EMPTY,"SHA-256");
        return sha+code;
    }

    @Override
    public InputStream encode(InputStream in) {
        if( !hasSecret() ){
            return in;
        }
        byte[] bytes = CommonIOUtils.inputStream2Bytes(in);
        try{
            String secret = getSecret();
            String data = new String(bytes,"UTF-8");
            data = TAG_START +codePassword()+TAG_END+
                    SecurityToolbox.aesEncrypt( data, secret);
            return new ByteArrayInputStream(data.getBytes("UTF-8"));
        }catch (Exception e){
            return new ByteArrayInputStream(bytes);
        }
    }

    @Override
    public InputStream decode(InputStream in) {
        byte[] bytes = CommonIOUtils.inputStream2Bytes(in);
        try{
            if( !Boolean.TRUE.equals(init.get()) ){
               initData(new ByteArrayInputStream(bytes));
            }
            if( !hasSecret() ){
                return new ByteArrayInputStream(bytes);
            }
            String data = new String(bytes,"UTF-8");
            data = data.substring( getPos() );
            data = SecurityToolbox.aesDecrypt( data, getSecret() );
            return new ByteArrayInputStream(data.getBytes("UTF-8"));
        }catch (Exception e){
            return new ByteArrayInputStream(bytes);
        }
    }

}
